--include "generator.lua"
--include "ETK/etk.lua"

bookmarks = {}

phrasePosition = math.random(1000000)

function gotoNextPhrase()
	phrasePosition = phrasePosition + 1
	math.randomseed(phrasePosition)
end

function gotoPreviousPhrase()
	if phrasePosition > 1 then
		phrasePosition = phrasePosition - 1
	end
	math.randomseed(phrasePosition)
end

function gotoRandomPhrase()
	phrasePosition = math.floor(timer.getMilliSecCounter()*math.random(10))
	math.randomseed(phrasePosition)
end

home = WScreen()

home.previousPhrase = sButton("Previous"    , gotoPreviousPhrase)
home.nextPhrase     = sButton("Next"        , gotoNextPhrase    )
home.randomPhrase   = sButton("Random"      , gotoRandomPhrase  )
home.addBookmark	= sButton("Bookmark", function () push_screen(addBookmark) end)
home.gotoBookmark   = sButton("Goto"    , function () push_screen(gotoBookmark) end)

home:appendWidget(home.previousPhrase, 10, -10)
    :appendWidget(home.nextPhrase    , 75, -10)
    :appendWidget(home.randomPhrase  , 115, -10)
	:appendWidget(home.addBookmark   , -50, -10)
	:appendWidget(home.gotoBookmark  , -10, -10)
	
home.nextPhrase:giveFocus()

function home:paint(gc)
	math.randomseed(phrasePosition)
	gc:setColorRGB(240,240,240)
	gc:fillRect(0, 0, self.w, self.h)
	
	gc:setColorRGB(0)

	
	gc:setFont("sansserif","b", 12)
	gc:drawString("Phrase #" .. phrasePosition .. (bookmarks[phrasePosition] and " (B: " .. bookmarks[phrasePosition] .. ")" or ""), 10, 2, "top")

	gc:setFont("sansserif","r", 10)
	local phrase = goNikky(" ", tables.rootTable)
	local words  = phrase:split()
	
	local spacesize = gc:getStringWidth(" ")
	local sentence = words[1]
	local sentences = 0
	
	for pos=2, #words do
		local word = words[pos]
		local possible = sentence .. words[pos] .. " "
		if gc:getStringWidth(possible)-spacesize >= self.w - 20 then
			gc:drawString(sentence, 10, sentences * 12 + 30, "top")
			sentence, sentences = word.." ", sentences + 1
		else
			sentence = possible
		end	
	end
	
	gc:drawString(sentence, 10, sentences * 12 + 30, "top")

	
	nativeBar(gc, self, self.h-45)
end






addBookmark	= Dialog("Add bookmark",50, 10, 240, 120)

addBookmark.txt	= sLabel ("Name for bookmark ")
addBookmark.inp = sInput()

function addBookmark:pushed()
	addBookmark.inp.value = "#" .. phrasePosition
	addBookmark.inp:giveFocus()
end

addBookmark.add	= sButton("Add")
addBookmark.can	= sButton("Cancel")

addBookmark:appendWidget(addBookmark.txt, 10,30)
           :appendWidget(addBookmark.inp,-10,30)
           :appendWidget(addBookmark.add,-65,-10)
           :appendWidget(addBookmark.can,-10,-10)

function addBookmark:postPaint(gc)
	nativeBar(gc, self, self.h-45)
end



function addBookmark.add:action()
	bookmarks[phrasePosition] = addBookmark.inp.value
	only_screen(home)
	document.markChanged()
end

function addBookmark.can:action()
	only_screen(home)
end

function addBookmark:escapeKey()
	only_screen(home)
end


function parseBookmarks()
	parsedBookmarks = {simple={}, links={}}
	for k, v in pairs(bookmarks) do
		table.insert(parsedBookmarks.simple, v)
		parsedBookmarks.links[v] = k
	end
	gotoBookmark.list.items = parsedBookmarks.simple
end

gotoBookmark	= Dialog("Goto bookmark", 50, 10, 240, 180)


gotoBookmark.list	= sList()

gotoBookmark:appendWidget(gotoBookmark.list,  5, 30)
gotoBookmark.list:setSize(-10, -35)
gotoBookmark.list:giveFocus()

function gotoBookmark:pushed()
	parseBookmarks()
end

function gotoBookmark.list:action(key, value)
	if value then
		phrasePosition = parsedBookmarks.links[value]
	end
	only_screen(home)
end

function gotoBookmark.list:backspaceKey()
	local w = self.items[self.sel]
	if w then
		bookmarks[parsedBookmarks.links[w]] = nil
		self:reset()
		parseBookmarks()
		document.markChanged()
	end
end


function gotoBookmark:escapeKey()
	only_screen(home)
end

function on.contextMenu()
	push_screen(gotoBookmark)
end

function on.save()
	return bookmarks
end

function on.restore(b)
	bookmarks = b
end	

push_screen(home)



